/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#include "Phoenix.h"
#ifndef _MFSERIAL_
#include "mfserial.h"
#endif

#ifdef POSIX
#include <termios.h>
#include <sys/ioctl.h>
#endif


MFSerialPort::MFSerialPort()
{
	#ifdef Linux
	tcgetattr(m_fdes, &m_comprmPrev);
	#endif

	#ifdef WIN32
	m_comprmPrev.DCBlength = sizeof(DCB);

	GetCommState(m_fdes, &m_comprmPrev);
	#endif
}
MFSerialPort::~MFSerialPort()
{
	#ifdef Linux
	tcsetattr(m_fdes, 0, &m_comprmPrev);	
	#endif

	#ifdef WIN32
	SetCommState(m_fdes, &m_comprmPrev);
	#endif
}


#if 0
#define ADD_BITS(v,m,b)		(v) = (((v) & ~(m))|(b))

Boolean MFSerialPort::SetSerialParams(MFSPBaud baud, int bytesize, MFSPParity parity, MFSPStopBits stop, unsigned flags)
{
	unsigned f;

	memset(&m_comprmNew, 0, sizeof(m_comprmNew));
	cfmakeraw(&m_comprmNew); // Clear it out to be a raw serial port.
	cfsetospeed(&m_comprmNew, baud); // Set to correct baud

	switch(bytesize)
	{
		case	5:
			f = CS5;
			break;
		case	6:
			f = CS6;
			break;
		case	7:
			f = CS7;
			break;
		case	8:
		default:
			f = CS8;
			break;
	}		
	
	ADD_BITS(m_comprmNew.c_cflag, CSIZE,f);

	switch(parity)
	{
		case 	MFSPParity_NONE:
			ADD_BITS(m_comprmNew.c_iflag,INPCK,0); 
			ADD_BITS(m_comprmNew.c_cflag,PARENB,0); 
			break;
		case	MFSPParity_ODD:
			ADD_BITS(m_comprmNew.c_iflag,INPCK,INPCK); 
			ADD_BITS(m_comprmNew.c_cflag,PARENB,PARENB); 
			ADD_BITS(m_comprmNew.c_cflag,PARODD,PARODD); 
			break;
		case	MFSPParity_EVEN:
			ADD_BITS(m_comprmNew.c_iflag,INPCK,0); 
			ADD_BITS(m_comprmNew.c_cflag,PARENB,0); 
			ADD_BITS(m_comprmNew.c_cflag,PARODD,0); 
			break;
	}

	if(stop == MFSPStopBits_2)
		 ADD_BITS(m_comprmNew.c_cflag, CSTOPB, CSTOPB);
	else
		 ADD_BITS(m_comprmNew.c_cflag, CSTOPB, 0);

	tcsetattr(m_fdes, 0, &m_comprmNew);	
	ioctl(m_fdes, TIOCM_DTR, 0);
	ioctl(m_fdes, TIOCM_CTS, 0);
   	return FALSE;
}
#endif

#ifdef WIN32
Boolean MFSerialPort::SetSerialParams(MFSPBaud baud, int bytesize, MFSPParity parity, MFSPStopBits stop, unsigned flags)
{
	DCB	dcb;
	COMMTIMEOUTS  CommTimeOuts ;

	char defstring[]="baud=9600 parity=N data=8 stop=1 xon=off rts=off dtr=off";

	if(m_fdes != (FILE_DESC) INVALID_FILE)
	{
		dcb.DCBlength = sizeof(dcb);

		if(GetCommState(m_fdes, &dcb))
		{
			SetCommMask(m_fdes, EV_RXCHAR);
			SetupComm(m_fdes, 4096, 4096);

			PurgeComm(m_fdes,	PURGE_TXABORT | PURGE_RXABORT |
								PURGE_TXCLEAR | PURGE_RXCLEAR ) ;

			BuildCommDCB(defstring, &dcb);

			dcb.BaudRate		= baud;
			dcb.ByteSize		= bytesize;
			dcb.fOutX			= (flags & MFSP_FLAGS_XON) ? 1 : 0;
			dcb.fInX			= dcb.fOutX;
			dcb.fBinary 		= TRUE;
			dcb.fParity 		= (parity == MFSPParity_NONE) ? FALSE : TRUE;
			dcb.fOutxDsrFlow	= FALSE;
			dcb.fOutxCtsFlow	= FALSE;
			dcb.fDtrControl 	= DTR_CONTROL_ENABLE;
			dcb.fRtsControl		= RTS_CONTROL_ENABLE;

			CommTimeOuts.ReadIntervalTimeout = 0xFFFFFFFF ;
			CommTimeOuts.ReadTotalTimeoutMultiplier = 0 ;
			CommTimeOuts.ReadTotalTimeoutConstant = 0 ;
			CommTimeOuts.WriteTotalTimeoutMultiplier = 20;
			CommTimeOuts.WriteTotalTimeoutConstant = 10000 ;
			SetCommTimeouts( m_fdes, &CommTimeOuts ) ;

			SetCommState(m_fdes, &dcb);

			EscapeCommFunction(m_fdes, SETDTR ) ;

			return TRUE;
		}

	}

	return FALSE;
}
#if 0
int NEAR ReadCommBlock( HANDLE hfile, LPSTR lpszBlock, int nMaxLength )
{
	BOOL		fReadStat ;
	COMSTAT    ComStat ;
	DWORD	   dwErrorFlags;
	DWORD	   dwLength;
	DWORD	   dwError;
	char	   szError[ 10 ] ;

	// only try to read number of bytes in queue
	ClearCommError( hfile, &dwErrorFlags, &ComStat ) ;
	dwLength = min( (DWORD) nMaxLength, ComStat.cbInQue ) ;

	if (dwLength > 0)
	{
		fReadStat = ReadFile( hfile, lpszBlock,
							dwLength, &dwLength, NULL) ;
		if (!fReadStat)
		{
            dwLength = 0 ;
			ClearCommError( hfile, &dwErrorFlags, &ComStat ) ;
			if (dwErrorFlags > 0)
			{
				printf("Com error %u\n", dwErrorFlags);
            }
		}
	}

	return ( dwLength ) ;

} // end of ReadCommBlock()

long MFSerialPort::read(void *s, long cb)
{
  return (m_fdes, s, (int)cb);

}
#endif
#endif
