/* 
   This code was not originated by me, but was acquired
   as-is. The original author was stripped out long before 
   I ever got it. MLW

   The changes and additions to this file to support C++ and newer versions
   of Linux are free to anyone who wishes to use them, subject to the terms 
   and conditions of the original authors license (If you can find it).
*/
#include "Phoenix.h"
#include "mfserial.h"
#include "unixcom.h"


#ifndef _POSIX
#  if defined (_BSD43) || defined (_V7)
static struct sgttyb sg;
static struct tchars tch;
static int lsw;
static int m_word;
#  endif
#endif

void MFSerialPort::sethwf(int on)
{
#ifdef _DGUX_SOURCE
  struct termiox x;
#endif
#ifdef _POSIX
  struct termios tty;

  tcgetattr(m_file, &tty);
  if (on)
	tty.c_cflag |= CRTSCTS;
  else
	tty.c_cflag &= ~CRTSCTS;
  tcsetattr(m_file, TCSANOW, &tty);
#endif

#ifdef _DGUX_SOURCE
  if (ioctl(m_file, TCGETX, &x) < 0) {
	mferr.mfprintf("can't get termiox attr.\n");
	return;
  }
  x.x_hflag = on ? RTSXOFF|CTSXON : 0;

  if (ioctl(m_file, TCSETX, &x) < 0) {
	mferr.mfprintf("can't set termiox attr.\n");
	return;
  }
#endif
}

void MFSerialPort::setrts()
{
#if defined(TIOCM_RTS) && defined(TIOCMODG)
  int mcs;

  ioctl(m_file, TIOCMODG, &mcs);
  mcs |= TIOCM_RTS;
  ioctl(m_file, TIOCMODS, &mcs);
#endif
#ifdef _COHERENT
  ioctl(m_file, TIOCSRTS, 0);
#endif
}

void MFSerialPort::dtrtoggle(int sec) 
{
#ifdef TIOCSDTR

  /* Use the ioctls meant for this type of thing. */
  ioctl(m_file, TIOCCDTR, 0);
  if (sec>0) {
    sleep(sec);
    ioctl(m_file, TIOCSDTR, 0);
  }

#else /* TIOCSDTR */
#  if defined (_POSIX) && !defined(_HPUX_SOURCE)

  /* Posix - set baudrate to 0 and back */
  struct termios tty, old;

  tcgetattr(m_file, &tty);
  tcgetattr(m_file, &old);
  cfsetospeed(&tty, B0);
  cfsetispeed(&tty, B0);
  tcsetattr(m_file, TCSANOW, &tty);
  if (sec>0) {
    sleep(sec);
    tcsetattr(m_file, TCSANOW, &old);
  }

#  else /* POSIX */
#    ifdef _V7

  /* Just drop speed to 0 and back to normal again */
  struct sgttyb sg, ng;
  
  ioctl(m_file, TIOCGETP, &sg);
  ioctl(m_file, TIOCGETP, &ng);
  
  ng.sg_ispeed = ng.sg_ospeed = 0;
  ioctl(m_file, TIOCSETP, &ng);
  if (sec>0) {
    sleep(sec);
    ioctl(m_file, TIOCSETP, &sg);
  }

#    endif /* _V7 */
#    ifdef _HPUX_SOURCE
  unsigned long mflag = 0L;

  ioctl(m_file, MCSETAF, &mflag);
  ioctl(m_file, MCGETA, &mflag);
  mflag = MRTS | MDTR;
  if (sec>0) {
    sleep(sec);
    ioctl(m_file, MCSETAF, &mflag);
  }
#    endif /* _HPUX_SOURCE */
#  endif /* POSIX */
#endif /* TIOCSDTR */
}

/*
 * Send a break
 */
void MFSerialPort::sendbreak()
{ 
#ifdef _POSIX
  tcsendbreak(m_file, 0);
#else
#  ifdef _V7
#    ifndef TIOCSBRK
  struct sgttyb sg, ng;

  ioctl(m_file, TIOCGETP, &sg);
  ioctl(m_file, TIOCGETP, &ng);
  ng.sg_ispeed = ng.sg_ospeed = B110;
  ng.sg_flags = BITS8 | RAW;
  ioctl(m_file, TIOCSETP, &ng);
  write(m_file, "\0\0\0\0\0\0\0\0\0\0", 10);
  ioctl(m_file, TIOCSETP, &sg);
#    else
  ioctl(m_file, TIOCSBRK, 0);
  sleep(1);
  ioctl(m_file, TIOCCBRK, 0);
#    endif
#  endif
#endif
}

/*
 * Get the dcd status
 */
int MFSerialPort::getdcd()
{
#ifdef TIOCMODG
  int mcs;
   
  ioctl(m_file, TIOCMODG, &mcs);
  return(mcs & TIOCM_CAR ? 1 : 0);
#else
  (void)m_file;
  return(0); /* Impossible!! (eg. Coherent) */
#endif
}


/*
 * Save the state of a port
 */
void MFSerialPort::savestate()
{
#ifdef _POSIX
  tcgetattr(m_file, &m_savetty);
#else
#  if defined(_BSD43) || defined(_V7)
  ioctl(m_file, TIOCGETP, &sg);
  ioctl(m_file, TIOCGETC, &tch);
#  endif
#  ifdef _BSD43
  ioctl(m_file, TIOCLGET, &lsw);
#  endif
#endif
#ifdef TIOCMODG
  ioctl(m_file, TIOCMODG, &m_word);
#endif
}

/*
 * Restore the state of a port
 */
void MFSerialPort::restorestate()
{
#ifdef _POSIX
  tcsetattr(m_file, TCSANOW, &m_savetty);
#else
#  if defined(_BSD43) || defined(_V7)
  ioctl(m_file, TIOCSETP, &sg);
  ioctl(m_file, TIOCSETC, &tch);
#  endif
#  ifdef _BSD43  
  ioctl(m_file, TIOCLSET, &lsw);
#  endif
#endif
#ifdef TIOCMODS
  ioctl(m_file, TIOCMODS, &m_word);
#endif
}

void MFSerialPort::nohang()
{
#ifdef _POSIX
  struct termios sgg;
  
  tcgetattr(m_file, &sgg);
  sgg.c_cflag |= CLOCAL;
  tcsetattr(m_file, TCSANOW, &sgg);
#else
#  if defined (_BSD43) && defined(LNOHANG)
  int lsw;
  
  ioctl(m_file, TIOCLGET, &lsw);
  lsw |= LNOHANG;
  ioctl(m_file, TIOCLSET, &lsw);
#  endif
#  ifdef _COHERENT
  /* Doesn't know about this either, me thinks. */
#  endif
#endif
}

void MFSerialPort::hupcl(int on)
{
  /* Eh, I don't know how to do this under BSD (yet..) */
#ifdef _POSIX
  struct termios sgg;
  
  tcgetattr(m_file, &sgg);
  if (on)
  	sgg.c_cflag |= HUPCL;
  else
	sgg.c_cflag &= ~HUPCL;
  tcsetattr(m_file, TCSANOW, &sgg);
#endif
}

/*
 * Flush the buffers
 */
void MFSerialPort::flush()
{
/* Should I Posixify this, or not? */
#ifdef TCFLSH
  ioctl(m_file, TCFLSH, (void *)2);
#endif
#ifdef TIOCFLUSH
#ifdef _COHERENT
  ioctl(m_file, TIOCFLUSH, 0);
#else
  ioctl(m_file, TIOCFLUSH, (void *)0);
#endif
#endif
}

int MFSerialPort::readchk()
{
#ifdef FIONREAD
  long i = -1;

  (void) ioctl(m_file, FIONREAD, &i);
  return((int)i);
#else
#  if defined(F_GETFL) && defined(O_NDELAY)
  int i, old;
  char c;

  old = fcntl(m_file, F_GETFL, 0);
  (void) fcntl(m_file, F_SETFL, old | O_NDELAY);
  i = read(m_file, &c, 1);
  (void) fcntl(m_file, F_SETFL, old);

  return(i);
#  else
  return(-1);
#  endif
#endif
}

/*
 * Get maximum speed.
 * Returns maximum speed / 100. (192-1152)
 */
int MFSerialPort::getmaxspd()
{
#ifdef B115200
  return(1152);
#elif defined(B57600)
  return(576);
#elif defined(B38400)
  return(384);
#elif defined(EXTB)
  return(384);
#elif defined(B19200)
  return(192);
#elif defined(EXTA)
  return(192);
#else
  return(96);
#endif
}

/*
 * Wait for child and return pid + status
 */
int MFSerialPort::comwait(int *stt)
{
#if defined (_BSD43) && !defined(_POSIX)
  int pid;
  union wait st1;
  
  pid = wait((void *)&st1);
  *stt = (unsigned)st1.w_retcode + 256 * (unsigned)st1.w_termsig;
  return(pid);
#else
  int pid;
  int st1;
  
  pid = wait(&st1);
  *stt = (unsigned)WEXITSTATUS(st1) + 256 * (unsigned)WTERMSIG(st1);
  return(pid);
#endif
}

/*
 * Set baudrate, parity and number of bits.
 */
void MFSerialPort::setparms(char *baudr, char *par, char *bits, int hwf, int swf)
{
  int spd = -1;
  int newbaud;
  int bit = bits[0];

#ifdef _POSIX
  struct termios tty;

  tcgetattr(m_file, &tty);
#else /* _POSIX */
  struct sgttyb tty;

  ioctl(m_file, TIOCGETP, &tty);
#endif /* _POSIX */

  /* We generate mark and space parity ourself. */
  if (bit == '7' && (par[0] == 'M' || par[0] == 'S'))
	bit = '8';

  /* Check if 'baudr' is really a number */
  if ((newbaud = (atol(baudr) / 100)) == 0 && baudr[0] != '0') newbaud = -1;

  switch(newbaud) {
  	case 0:
#ifdef B0
			spd = B0;	break;
#else
			spd = 0;	break;
#endif
  	case 3:		spd = B300;	break;
  	case 6:		spd = B600;	break;
  	case 12:	spd = B1200;	break;
  	case 24:	spd = B2400;	break;
  	case 48:	spd = B4800;	break;
  	case 96:	spd = B9600;	break;
#ifdef B19200
  	case 192:	spd = B19200;	break;
#else /* B19200 */
#  ifdef EXTA
	case 192:	spd = EXTA;	break;
#   else /* EXTA */
	case 192:	spd = B9600;	break;
#   endif /* EXTA */
#endif	 /* B19200 */
#ifdef B38400
  	case 384:	spd = B38400;	break;
#else /* B38400 */
#  ifdef EXTB
	case 384:	spd = EXTB;	break;
#   else /* EXTB */
	case 384:	spd = B9600;	break;
#   endif /* EXTB */
#endif	 /* B38400 */
#ifdef B57600
	case 576:	spd = B57600;	break;
#endif
#ifdef B115200
	case 1152:	spd = B115200;	break;
#endif
  }
  
#if defined (_BSD43) && !defined(_POSIX)
  if (spd != -1) tty.sg_ispeed = tty.sg_ospeed = spd;
  /* Number of bits is ignored */

  tty.sg_flags = RAW | TANDEM;
  if (par[0] == 'E')
	tty.sg_flags |= EVENP;
  else if (par[0] == 'O')
	tty.sg_flags |= ODDP;
  else
  	tty.sg_flags |= PASS8 | ANYP;

  ioctl(m_file, TIOCSETP, &tty);

#  ifdef TIOCSDTR
  /* FIXME: huh? - MvS */
  ioctl(m_file, TIOCSDTR, 0);
#  endif
#endif /* _BSD43 && !_POSIX */

#if defined (_V7) && !defined(_POSIX)
  if (spd != -1) tty.sg_ispeed = tty.sg_ospeed = spd;
  tty.sg_flags = RAW;
  if (par[0] == 'E')
	tty.sg_flags |= EVENP;
  else if (par[0] == 'O')
	tty.sg_flags |= ODDP;

  ioctl(m_file, TIOCSETP, &tty);
#endif /* _V7 && !POSIX */

#ifdef _POSIX

  if (spd != -1) {
	cfsetospeed(&tty, (speed_t)spd);
	cfsetispeed(&tty, (speed_t)spd);
  }

  switch (bit) {
  	case '5':
  		tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS5;
  		break;
  	case '6':
  		tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS6;
  		break;
  	case '7':
  		tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS7;
  		break;
  	case '8':
	default:
  		tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;
  		break;
  }		
  /* Set into raw, no echo mode */
  tty.c_iflag =  IGNBRK;
  tty.c_lflag = 0;
  tty.c_oflag = 0;
  tty.c_cflag |= CLOCAL | CREAD;
#ifdef _DCDFLOW
  tty.c_cflag &= ~CRTSCTS;
#endif
  tty.c_cc[VMIN] = 1;
  tty.c_cc[VTIME] = 5;

  if (swf)
	tty.c_iflag |= IXON | IXOFF;
  else
	tty.c_iflag &= ~(IXON|IXOFF|IXANY);

  tty.c_cflag &= ~(PARENB | PARODD);
  if (par[0] == 'E')
	tty.c_cflag |= PARENB;
  else if (par[0] == 'O')
	tty.c_cflag |= PARODD;

  tcsetattr(m_file, TCSANOW, &tty);

  setrts();
#endif /* _POSIX */

#ifndef _DCDFLOW
  sethwf(hwf);
#endif
}


/*
 * Set baudrate, parity and number of bits.
 */
// void MFSerialPort::setparms(int baud, int par, int bits, int hwf, int swf)
Boolean MFSerialPort::SetSerialParams(MFSPBaud baud, int bits, MFSPParity par, MFSPStopBits stop, unsigned flags)
{

#ifdef _POSIX
  	struct termios tty;

  	tcgetattr(m_file, &tty);
#else /* _POSIX */
  	struct sgttyb tty;

  	ioctl(m_file, TIOCGETP, &tty);
#endif /* _POSIX */

  	/* We generate mark and space parity ourself. */
  	if (bits == 7 && (par == MFSPParity_MARK || par == MFSPParity_SPACE))
		bits = 8;

#if defined (_BSD43) && !defined(_POSIX)
  	if (baud != -1) 
		tty.sg_ispeed = tty.sg_ospeed = baud;

  	tty.sg_flags = RAW | TANDEM;
  	if (par == MFSPParity_EVEN)
		tty.sg_flags |= EVENP;
 	 else if (par == MFSPParity_ODD)
		tty.sg_flags |= ODDP;
  	else
  		tty.sg_flags |= PASS8 | ANYP;

  	ioctl(m_file, TIOCSETP, &tty);

#  ifdef TIOCSDTR
  /* FIXME: huh? - MvS */
  	ioctl(m_file, TIOCSDTR, 0);
#  endif
#endif /* _BSD43 && !_POSIX */

#if defined (_V7) && !defined(_POSIX)
  	if (baud != -1) 
		tty.sg_ispeed = tty.sg_ospeed = baud;

  	tty.sg_flags = RAW;

  	if (par[0] == 'E')
		tty.sg_flags |= EVENP;
  	else if (par[0] == 'O')
		tty.sg_flags |= ODDP;

  	ioctl(m_file, TIOCSETP, &tty);
#endif /* _V7 && !POSIX */

#ifdef _POSIX

  	if (baud != -1) 
	{
		cfsetospeed(&tty, (speed_t)baud);
		cfsetispeed(&tty, (speed_t)baud);
  	}

  	switch (bits) 
	{
  		case 5:
  			tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS5;
  			break;
  		case 6:
  			tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS6;
  			break;
  		case 7:
  			tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS7;
  			break;
  		case 8:
		default:
  			tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;
  			break;
  	}
		
	if(stop == MFSPStopBits_2)
		tty.c_cflag = tty.c_cflag | CSTOPB;

  	tty.c_iflag =  IGNBRK;
  	tty.c_lflag = 0;
  	tty.c_oflag = 0;
  	tty.c_cflag |= CLOCAL | CREAD;

#ifdef _DCDFLOW
  	tty.c_cflag &= ~CRTSCTS;
#endif

  	tty.c_cc[VMIN] = 1;
  	tty.c_cc[VTIME] = 5;

	tty.c_iflag &= ~(IXON|IXOFF|IXANY);

  	tty.c_cflag &= ~(PARENB | PARODD);

  	if (par == MFSPParity_EVEN)
		tty.c_cflag |= PARENB;
 	 else if (par == MFSPParity_ODD)
		tty.c_cflag |= PARODD;

  	tcsetattr(m_file, TCSANOW, &tty);

  	setrts();
#endif /* _POSIX */

#ifndef _DCDFLOW
  	sethwf(0);
#endif
	return TRUE;
}

